@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-18 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -121,13 +121,7 @@ public:
 	{
 		return CVector3(x / scalar, y / scalar, z / scalar);
 	}
-	inline CVector3 &operator=(const CVector3 &vector)
-	{
-		x = vector.x;
-		y = vector.y;
-		z = vector.z;
-		return *this;
-	}
+	inline CVector3 &operator=(const CVector3 &vector) = default;
 	inline CVector3 &operator+=(const CVector3 &vector)
 	{
 		x += vector.x;
@@ -334,14 +328,7 @@ public:
 	}
 
 	inline CVector4 operator/(const double &s) const { return CVector4(x / s, y / s, z / s, w / s); }
-	inline CVector4 &operator=(const CVector4 &v)
-	{
-		x = v.x;
-		y = v.y;
-		z = v.z;
-		w = v.w;
-		return *this;
-	}
+	inline CVector4 &operator=(const CVector4 &v) = default;
 	inline CVector4 &operator+=(const CVector4 &v)
 	{
 		x += v.x;
@@ -410,7 +397,7 @@ public:
 		v.x = x;
 		v.y = y;
 		v.z = z;
-		wOutput = this->w;
+		wOutput = w;
 	}
 	inline CVector3 GetXYZ() const { return CVector3(x, y, z); }
 
