@@ -61,6 +61,10 @@
 #define M_PI_4 0.78539816339744830961566084581988
 #endif
 
+#ifndef M_PI_2
+#define M_PI_2 1.57079632679489661923132169163975
+#endif
+
 #ifndef M_PI_2x
 #define M_PI_2x 6.283185307179586476925286766559
 #endif
@@ -83,6 +87,7 @@ void AboxMod13Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux)
 void AboxModKaliIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AboxModKaliEiffieIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AboxModKaliV2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void AboxModKaliV3Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AboxVSIcen1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AexionIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void AexionOctopusModIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -107,11 +112,11 @@ void CollatzModIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux
 void CrossMengerMod1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void EiffieMsltoeIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void FoldBoxMod1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
-void ImaginaryScatorPower2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void GeneralizedFoldBoxIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void HypercomplexIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Ides2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void IdesIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
 void IqBulbIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void JosKleinianIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void KaleidoscopicIfsIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -168,9 +173,15 @@ void QuaternionIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux
 void QuaternionCubic4dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void QuickDudleyIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void QuickDudleyModIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
 void RiemannSphereMsltoeIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void RiemannSphereMsltoeV1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void RiemannBulbMsltoeMod2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
+void ScatorPower2ImaginaryIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void ScatorPower2RealIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void ScatorPower2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void ScatorPower2StdRIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void Sierpinski3dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void MandelboxSmoothIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void XenodreambuieIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -182,6 +193,8 @@ void TransfAddCpixelAxisSwapIteration(CVector4 &z, const sFractal *fractal, sExt
 void TransfAddCpixelCxCyAxisSwapIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfAddCpixelIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfAddCpixelRotatedIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void TransfAddCpixelScatorIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void TransfAddCpixelSinOrCosIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfAddCpixelSymmetricalIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfAddCpixelVaryV1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfAddConstantIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -198,10 +211,13 @@ void TransfBenesiT2Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux
 void TransfBenesiT3Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfBenesiT4Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfBenesiT5bIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void TransfBlockifyIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
 void TransfBoxFoldIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfBoxFoldVaryV1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfBoxFoldXYZIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfBoxOffsetIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
 void TransfAbsAddConstantIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfAbsAddTgladFoldIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfAbsAddConditionalIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -234,6 +250,11 @@ void TransfScaleVaryMultiIteration(CVector4 &z, const sFractal *fractal, sExtend
 void TransfScaleVaryV1Iteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfScaleVaryVCLIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfScale3dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
+void TransfSinOrCosIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void TransfSinAndCosIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+void TransfSinAndCosMaxIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
 void TransfSphericalInvCIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfSphericalInvIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 void TransfSphericalFoldIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
@@ -281,4 +302,6 @@ void TestingIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 
 void Testing4dIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
 
+void TestingLogIteration(CVector4 &z, const sFractal *fractal, sExtendedAux &aux);
+
 #endif /* MANDELBULBER2_SRC_FRACTAL_FORMULAS_HPP_ */
