@@ -52,6 +52,7 @@
 #include "nine_fractals.hpp"
 #include "opencl_dynamic_data.hpp"
 #include "opencl_hardware.h"
+#include "opencl_textures_data.h"
 #include "parameters.hpp"
 #include "progress_text.hpp"
 #include "rectangle.hpp"
@@ -63,26 +64,19 @@
 #ifdef USE_OPENCL
 #include "opencl/fractal_cl.h"
 #include "opencl/fractparams_cl.hpp"
+#include "../opencl/mesh_export_data_cl.h"
 #endif
 
 cOpenClEngineRenderFractal::cOpenClEngineRenderFractal(cOpenClHardware *_hardware)
 		: cOpenClEngine(_hardware)
 {
 #ifdef USE_OPENCL
-	constantInBuffer = nullptr;
-	inCLConstBuffer = nullptr;
-	backgroungImageBuffer = nullptr;
-	backgroundImage2D = nullptr;
-
-	inCLBuffer = nullptr;
-
-	dynamicData = new cOpenClDynamicData;
-
 	optimalJob.sizeOfPixel = sizeof(sClPixel);
 	autoRefreshMode = false;
 	monteCarlo = false;
 
 	renderEngineMode = clRenderEngineTypeNone;
+	meshExportMode = false;
 
 #endif
 }
@@ -98,16 +92,20 @@ cOpenClEngineRenderFractal::~cOpenClEngineRenderFractal()
 
 void cOpenClEngineRenderFractal::ReleaseMemory()
 {
-	if (constantInBuffer) delete constantInBuffer;
-	constantInBuffer = nullptr;
-	if (inCLConstBuffer) delete inCLConstBuffer;
-	inCLConstBuffer = nullptr;
-	if (inCLBuffer) delete inCLBuffer;
-	inCLBuffer = nullptr;
-	if (backgroungImageBuffer) delete backgroungImageBuffer;
-	backgroungImageBuffer = nullptr;
+	constantInBuffer.reset();
+	inCLConstBuffer.reset();
+	constantInMeshExportBuffer.reset();
+	inCLConstMeshExportBuffer.reset();
+	inCLBuffer.reset();
+	inCLTextureBuffer.reset();
+	backgroundImage2D.reset();
+	backgroungImageBuffer.reset();
+	dynamicData.reset();
+	texturesData.reset();
+	inBuffer.clear();
+	inTextureBuffer.clear();
 
-	dynamicData->Clear();
+	cOpenClEngine::ReleaseMemory();
 }
 
 QString cOpenClEngineRenderFractal::GetKernelName()
@@ -138,13 +136,17 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 		clHeaderFiles.append("fractal_coloring_cl.hpp");
 		clHeaderFiles.append("fractparams_cl.hpp");
 		clHeaderFiles.append("fractal_sequence_cl.h");
+		clHeaderFiles.append("texture_enums_cl.h");
 		clHeaderFiles.append("material_cl.h");
+		clHeaderFiles.append("shader_input_data_cl.h");
 		clHeaderFiles.append("object_type_cl.h");
 		clHeaderFiles.append("object_data_cl.h");
 		clHeaderFiles.append("primitives_cl.h");
 		clHeaderFiles.append("input_data_structures.h");
 		clHeaderFiles.append("render_data_cl.h");
 
+		if (meshExportMode) clHeaderFiles.append("mesh_export_data_cl.h");
+
 		// pass through define constants
 		programEngine.append("#define USE_OPENCL 1\n");
 
@@ -187,44 +189,95 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 		// compute fractal
 		programEngine.append("#include \"" + openclEnginePath + "compute_fractal.cl\"\n");
 
+		if (!meshExportMode || true)
+		{
+			// texture mapping
+			programEngine.append("#include \"" + openclEnginePath + "texture_mapping.cl\"\n");
+
+			if (renderEngineMode != clRenderEngineTypeFast)
+			{
+				// calculation of texture pixel address
+				programEngine.append(
+					"#include \"" + openclEnginePath + "shader_texture_pixel_address.cl\"\n");
+
+				// calculation of bicubic interpolation for textures
+				programEngine.append("#include \"" + openclEnginePath + "bicubic_interpolation.cl\"\n");
+
+				// calculate displacement from textures
+				programEngine.append("#include \"" + openclEnginePath + "displacement_map.cl\"\n");
+			}
+		}
+
 		// compute fractal
 		programEngine.append("#include \"" + openclEnginePath + "primitives.cl\"\n");
 
 		// calculate distance
 		programEngine.append("#include \"" + openclEnginePath + "calculate_distance.cl\"\n");
 
-		// normal vector calculation
-		programEngine.append("#include \"" + openclEnginePath + "normal_vector.cl\"\n");
-
-		// 3D projections (3point, equirectagular, fisheye)
-		programEngine.append("#include \"" + openclEnginePath + "projection_3d.cl\"\n");
-
-		if (renderEngineMode != clRenderEngineTypeFast)
+		if (!meshExportMode || true)
 		{
-			// shaders
-			programEngine.append("#include \"" + openclEnginePath + "shaders.cl\"\n");
-		}
+			// normal vector calculation
+			programEngine.append("#include \"" + openclEnginePath + "normal_vector.cl\"\n");
 
-		if (renderEngineMode == clRenderEngineTypeFull)
-		{
-			// ray recursion
-			programEngine.append("#include \"" + openclEnginePath + "ray_recursion.cl\"\n");
-		}
+			// 3D projections (3point, equirectagular, fisheye)
+			programEngine.append("#include \"" + openclEnginePath + "projection_3d.cl\"\n");
 
-		if (renderEngineMode != clRenderEngineTypeFast)
-		{
-			// Monte Carlo DOF
-			programEngine.append("#include \"" + openclEnginePath + "monte_carlo_dof.cl\"\n");
+			if (renderEngineMode != clRenderEngineTypeFast)
+			{
+				// shaders
+				programEngine.append("#include \"" + openclEnginePath + "shader_iter_opacity.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_hsv2rgb.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_background.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_surface_color.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_main_shading.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_specular_highlight.cl\"\n");
+				programEngine.append(
+					"#include \"" + openclEnginePath + "shader_specular_highlight_combined.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_main_shadow.cl\"\n");
+				programEngine.append(
+					"#include \"" + openclEnginePath + "shader_fast_ambient_occlusion.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_ambient_occlusion.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_aux_shadow.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_light_shading.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_aux_lights_shader.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_fake_lights.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_iridescence.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_texture.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_normal_map_texture.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_object.cl\"\n");
+				programEngine.append(
+					"#include \"" + openclEnginePath + "shader_global_illumination.cl\"\n");
+				programEngine.append("#include \"" + openclEnginePath + "shader_volumetric.cl\"\n");
+			}
+
+			if (renderEngineMode == clRenderEngineTypeFull)
+			{
+				// ray recursion
+				programEngine.append("#include \"" + openclEnginePath + "ray_recursion.cl\"\n");
+			}
+
+			if (renderEngineMode != clRenderEngineTypeFast)
+			{
+				// Monte Carlo DOF
+				programEngine.append("#include \"" + openclEnginePath + "monte_carlo_dof.cl\"\n");
+			}
 		}
 
 		// main engine
 		QString engineFileName;
-		switch (renderEngineMode)
+		if (meshExportMode)
 		{
-			case clRenderEngineTypeFast: engineFileName = "fast_engine.cl"; break;
-			case clRenderEngineTypeLimited: engineFileName = "limited_engine.cl"; break;
-			case clRenderEngineTypeFull: engineFileName = "full_engine.cl"; break;
-			case clRenderEngineTypeNone: break;
+			engineFileName = "slicer_engine.cl";
+		}
+		else
+		{
+			switch (renderEngineMode)
+			{
+				case clRenderEngineTypeFast: engineFileName = "fast_engine.cl"; break;
+				case clRenderEngineTypeLimited: engineFileName = "limited_engine.cl"; break;
+				case clRenderEngineTypeFull: engineFileName = "full_engine.cl"; break;
+				case clRenderEngineTypeNone: break;
+			}
 		}
 		QString engineFullFileName = openclEnginePath + engineFileName;
 		programEngine.append(LoadUtf8TextFromFile(engineFullFileName));
@@ -260,12 +313,18 @@ bool cOpenClEngineRenderFractal::LoadSourcesAndCompile(const cParameterContainer
 
 void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramContainer,
 	const cFractalContainer *fractalContainer, sParamRender *paramRender, cNineFractals *fractals,
-	sRenderData *renderData)
+	sRenderData *renderData, bool meshExportModeEnable)
 {
 	Q_UNUSED(fractalContainer);
 
-	if (constantInBuffer) delete constantInBuffer;
-	constantInBuffer = new sClInConstants;
+	meshExportMode = meshExportModeEnable;
+
+	constantInBuffer.reset(new sClInConstants);
+
+	if (meshExportMode)
+	{
+		constantInMeshExportBuffer.reset(new sClMeshExport);
+	}
 
 	definesCollector.clear();
 
@@ -278,6 +337,8 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 
 	if (renderEngineMode == clRenderEngineTypeFull) definesCollector += " -DFULL_ENGINE";
 
+	if (meshExportMode) definesCollector += " -DMESH_EXPORT";
+
 	// define distance estimation method
 	fractal::enumDEType deType = fractals->GetDEType(0);
 	fractal::enumDEFunctionType deFunctionType = fractals->GetDEFunctionType(0);
@@ -423,9 +484,9 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	if (paramRender->booleanOperatorsEnabled) definesCollector += " -DBOOLEAN_OPERATORS";
 
 	bool isVolumetricLight = false;
-	for (int i = 0; i < 5; i++)
+	for (bool i : paramRender->volumetricLightEnabled)
 	{
-		if (paramRender->volumetricLightEnabled[i]) isVolumetricLight = true;
+		if (i) isVolumetricLight = true;
 	}
 	if (isVolumetricLight)
 	{
@@ -456,8 +517,30 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	}
 	if (!anyVolumetricShaderUsed) definesCollector += " -DSIMPLE_GLOW";
 
-	if (paramRender->DOFMonteCarlo && paramRender->DOFEnabled)
-		definesCollector += " -DMONTE_CARLO_DOF";
+	if (paramRender->DOFMonteCarlo)
+	{
+		definesCollector += " -DMONTE_CARLO";
+
+		if (paramRender->DOFEnabled)
+		{
+			definesCollector += " -DMONTE_CARLO_DOF";
+		}
+
+		definesCollector += " -DMONTE_CARLO_ANTI_ALIASING";
+
+		if (paramRender->DOFMonteCarloGlobalIllumination)
+		{
+			definesCollector += " -DMONTE_CARLO_DOF_GLOBAL_ILLUMINATION";
+		}
+		if (paramRender->DOFMonteCarloChromaticAberration)
+		{
+			definesCollector += " -DCHROMATIC_ABERRATION";
+		}
+		if (paramRender->monteCarloSoftShadows)
+		{
+			definesCollector += " -DMC_SOFT_SHADOWS";
+		}
+	}
 
 	if (paramRender->texturedBackground)
 	{
@@ -476,14 +559,35 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 
 	WriteLogDouble("Constant buffer size [KB]", sizeof(sClInConstants) / 1024.0, 3);
 
+	renderData->ValidateObjects();
+
+	QMap<QString, int> textureIndexes;
+
+	// --------------- textures dynamic data -----------------
+	if (renderEngineMode == clRenderEngineTypeFull)
+	{
+		int numberOfTextures =
+			cOpenClTexturesData::CheckNumberOfTextures(renderData->textures, renderData->materials);
+
+		texturesData.reset(new cOpenClTexturesData(numberOfTextures));
+		texturesData->ReserveHeader();
+		texturesData->BuildAllTexturesData(
+			renderData->textures, renderData->materials, &textureIndexes);
+		texturesData->FillHeader();
+		inTextureBuffer = texturesData->GetData();
+
+		if (numberOfTextures > 0) definesCollector += " -DUSE_TEXTURES";
+		definesCollector += " -DNUMBER_OF_TEXTURES=" + QString::number(numberOfTextures);
+
+		definesCollector += texturesData->GetDefinesCollector();
+	}
+
 	//----------- create dynamic data -----------
-	dynamicData->Clear();
+	dynamicData.reset(new cOpenClDynamicData);
 	dynamicData->ReserveHeader();
 
 	// materials
-	QMap<int, cMaterial> materials;
-	CreateMaterialsMap(paramContainer, &materials, true);
-	int materialsArraySize = dynamicData->BuildMaterialsData(materials);
+	int materialsArraySize = dynamicData->BuildMaterialsData(renderData->materials, textureIndexes);
 	definesCollector += " -DMAT_ARRAY_SIZE=" + QString::number(materialsArraySize);
 
 	bool anyMaterialIsReflective = false;
@@ -492,7 +596,8 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	bool anyMaterialHasIridescence = false;
 	bool anyMaterialHasColoringEnabled = false;
 	bool anyMaterialHasExtraColoringEnabled = false;
-	foreach (cMaterial material, materials)
+	bool anyMaterialHasTextureFractalize = false;
+	foreach (cMaterial material, renderData->materials)
 	{
 		if (material.reflectance > 0.0) anyMaterialIsReflective = true;
 		if (material.transparencyOfSurface > 0.0) anyMaterialIsRefractive = true;
@@ -501,6 +606,7 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 		if (material.iridescenceEnabled) anyMaterialHasIridescence = true;
 		if (material.useColorsFromPalette) anyMaterialHasColoringEnabled = true;
 		if (material.fractalColoring.extraColorEnabledFalse) anyMaterialHasExtraColoringEnabled = true;
+		if (material.textureFractalize) anyMaterialHasTextureFractalize = true;
 	}
 	if (anyMaterialIsReflective) definesCollector += " -DUSE_REFLECTANCE";
 	if (anyMaterialIsRefractive) definesCollector += " -DUSE_REFRACTION";
@@ -510,7 +616,8 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 		definesCollector += " -DUSE_FRACTAL_COLORING";
 	if (anyMaterialHasExtraColoringEnabled) definesCollector += " -DUSE_EXTRA_COLORING";
 
-	if ((anyMaterialIsReflective || anyMaterialIsRefractive) && paramRender->raytracedReflections)
+	if (((anyMaterialIsReflective || anyMaterialIsRefractive) && paramRender->raytracedReflections)
+			|| paramRender->DOFMonteCarloGlobalIllumination)
 	{
 		definesCollector += " -DREFLECTIONS_MAX=" + QString::number(paramRender->reflectionsMax + 1);
 	}
@@ -519,6 +626,7 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 		paramRender->reflectionsMax = 0;
 		definesCollector += " -DREFLECTIONS_MAX=1";
 	}
+	if (anyMaterialHasTextureFractalize) definesCollector += " -DFRACTALIZE_TEXTURE";
 
 	// AO colored vectors
 	cRenderWorker *tempRenderWorker =
@@ -529,7 +637,7 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 	dynamicData->BuildAOVectorsData(AOVectors, numberOfVectors);
 	dynamicData->BuildLightsData(&renderData->lights);
 
-	dynamicData->BuildPrimitivesData(&paramRender->primitives);
+	definesCollector += dynamicData->BuildPrimitivesData(&paramRender->primitives);
 	if (paramRender->primitives.GetListOfPrimitives()->size() > 0)
 		definesCollector += " -DUSE_PRIMITIVES";
 
@@ -542,8 +650,7 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 
 	//---------------- another parameters -------------
 	autoRefreshMode = paramContainer->Get<bool>("auto_refresh");
-	monteCarlo = paramRender->DOFMonteCarlo && paramRender->DOFEnabled
-							 && renderEngineMode != clRenderEngineTypeFast;
+	monteCarlo = paramRender->DOFMonteCarlo && renderEngineMode != clRenderEngineTypeFast;
 
 	// copy all cl parameters to constant buffer
 	constantInBuffer->params = clCopySParamRenderCl(*paramRender);
@@ -563,7 +670,22 @@ void cOpenClEngineRenderFractal::SetParameters(const cParameterContainer *paramC
 void cOpenClEngineRenderFractal::RegisterInputOutputBuffers(const cParameterContainer *params)
 {
 	Q_UNUSED(params);
-	outputBuffers << sClInputOutputBuffer(sizeof(sClPixel), optimalJob.stepSize, "output-buffer");
+	if (!meshExportMode)
+	{
+		outputBuffers << sClInputOutputBuffer(sizeof(sClPixel), optimalJob.stepSize, "output-buffer");
+	}
+	else
+	{
+		// buffer for distances
+		outputBuffers << sClInputOutputBuffer(sizeof(float),
+			constantInMeshExportBuffer->sliceHeight * constantInMeshExportBuffer->sliceWidth,
+			"mesh-distances-buffer");
+
+		// buffer for colors
+		outputBuffers << sClInputOutputBuffer(sizeof(float),
+			constantInMeshExportBuffer->sliceHeight * constantInMeshExportBuffer->sliceWidth,
+			"mesh-colors-buffer");
+	}
 }
 
 bool cOpenClEngineRenderFractal::PreAllocateBuffers(const cParameterContainer *params)
@@ -577,9 +699,9 @@ bool cOpenClEngineRenderFractal::PreAllocateBuffers(const cParameterContainer *p
 	if (hardware->ContextCreated())
 	{
 
-		if (inCLConstBuffer) delete inCLConstBuffer;
-		inCLConstBuffer = new cl::Buffer(*hardware->getContext(),
-			CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, sizeof(sClInConstants), constantInBuffer, &err);
+		inCLConstBuffer.reset(
+			new cl::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
+				sizeof(sClInConstants), constantInBuffer.data(), &err));
 		if (!checkErr(err,
 					"cl::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, "
 					"sizeof(sClInConstants), constantInBuffer, &err)"))
@@ -590,10 +712,25 @@ bool cOpenClEngineRenderFractal::PreAllocateBuffers(const cParameterContainer *p
 			return false;
 		}
 
+		if (meshExportMode)
+		{
+			inCLConstMeshExportBuffer.reset(
+				new cl::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
+					sizeof(sClMeshExport), constantInMeshExportBuffer.data(), &err));
+			if (!checkErr(err,
+						"cl::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, "
+						"sizeof(inCLConstMeshExportBuffer), constantInBuffer, &err)"))
+			{
+				emit showErrorMessage(
+					QObject::tr("OpenCL %1 cannot be created!").arg(QObject::tr("buffer for constants")),
+					cErrorMessage::errorMessage, nullptr);
+				return false;
+			}
+		}
+
 		// this buffer will be used for color palettes, lights, etc...
-		if (inCLBuffer) delete inCLBuffer;
-		inCLBuffer = new cl::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
-			size_t(inBuffer.size()), inBuffer.data(), &err);
+		inCLBuffer.reset(new cl::Buffer(*hardware->getContext(),
+			CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR, size_t(inBuffer.size()), inBuffer.data(), &err));
 		if (!checkErr(err,
 					"Buffer::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_USE_HOST_PTR, "
 					"sizeof(sClInBuff), inBuffer, &err)"))
@@ -603,7 +740,25 @@ bool cOpenClEngineRenderFractal::PreAllocateBuffers(const cParameterContainer *p
 				cErrorMessage::errorMessage, nullptr);
 			return false;
 		}
+
+		if (renderEngineMode == clRenderEngineTypeFull)
+		{
+			// this buffer will be used for textures
+			inCLTextureBuffer.reset(
+				new cl::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
+					size_t(inTextureBuffer.size()), inTextureBuffer.data(), &err));
+			if (!checkErr(err,
+						"Buffer::Buffer(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_USE_HOST_PTR, "
+						"sizeof(sClInBuff), inBuffer, &err)"))
+			{
+				emit showErrorMessage(
+					QObject::tr("OpenCL %1 cannot be created!").arg(QObject::tr("buffer for texture data")),
+					cErrorMessage::errorMessage, nullptr);
+				return false;
+			}
+		}
 	}
+
 	else
 	{
 		emit showErrorMessage(
@@ -628,7 +783,9 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 		cProgressText progressText;
 		progressText.ResetTimer();
 
-		emit updateProgressAndStatus(tr("OpenCl - rendering image"), progressText.getText(0.0), 0.0);
+		emit updateProgressAndStatus(
+			tr("OpenCl - rendering image (workgroup %1 pixels)").arg(optimalJob.workGroupSize),
+			progressText.getText(0.0), 0.0);
 
 		QElapsedTimer timer;
 		timer.start();
@@ -764,7 +921,7 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 								for (int y = 0; y < jobHeight2; y++)
 								{
 									sClPixel pixelCl =
-										((sClPixel *)outputBuffers[outputIndex].ptr)[x + y * jobWidth2];
+										((sClPixel *)outputBuffers[outputIndex].ptr.data())[x + y * jobWidth2];
 									sRGBFloat pixel = {pixelCl.R, pixelCl.G, pixelCl.B};
 									sRGB8 color = {pixelCl.colR, pixelCl.colG, pixelCl.colB};
 									unsigned short opacity = pixelCl.opacity;
@@ -796,6 +953,10 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 																	 + (newPixel.G - oldPixel.G) * (newPixel.G - oldPixel.G)
 																	 + (newPixel.B - oldPixel.B) * (newPixel.B - oldPixel.B);
 										noise *= 0.3333;
+
+										double sumBrightness = newPixel.R + newPixel.G + newPixel.B;
+										if (sumBrightness > 1.0) noise /= (sumBrightness * sumBrightness);
+
 										monteCarloNoiseSum += noise;
 										if (noise > maxNoise) maxNoise = noise;
 									}
@@ -841,7 +1002,8 @@ bool cOpenClEngineRenderFractal::Render(cImage *image, bool *stopRequest, sRende
 								percentDone = percentDone * (1.0 - doneMC) + doneMC;
 							}
 							emit updateProgressAndStatus(
-								tr("OpenCl - rendering image"), progressText.getText(percentDone), percentDone);
+								tr("OpenCl - rendering image (workgroup %1 pixels)").arg(optimalJob.workGroupSize),
+								progressText.getText(percentDone), percentDone);
 
 							emit updateStatistics(renderData->statistics);
 
@@ -954,7 +1116,7 @@ QList<QPoint> cOpenClEngineRenderFractal::calculateOptimalTileSequence(
 	}
 	qSort(tiles.begin(), tiles.end(),
 		std::bind(cOpenClEngineRenderFractal::sortByCenterDistanceAsc, std::placeholders::_1,
-			std::placeholders::_2, gridWidth, gridHeight));
+					std::placeholders::_2, gridWidth, gridHeight));
 	return tiles;
 }
 
@@ -1046,6 +1208,18 @@ bool cOpenClEngineRenderFractal::AssignParametersToKernelAdditional(int argItera
 		return false;
 	}
 
+	if (!meshExportMode && renderEngineMode == clRenderEngineTypeFull)
+	{
+		int err = kernel->setArg(argIterator++, *inCLTextureBuffer); // input data in global memory
+		if (!checkErr(err, "kernel->setArg(1, *inCLTextureBuffer)"))
+		{
+			emit showErrorMessage(
+				QObject::tr("Cannot set OpenCL argument for %1").arg(QObject::tr("input texture data")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
+	}
+
 	err = kernel->setArg(
 		argIterator++, *inCLConstBuffer); // input data in constant memory (faster than global)
 	if (!checkErr(err, "kernel->setArg(2, *inCLConstBuffer)"))
@@ -1056,7 +1230,20 @@ bool cOpenClEngineRenderFractal::AssignParametersToKernelAdditional(int argItera
 		return false;
 	}
 
-	if (renderEngineMode != clRenderEngineTypeFast)
+	if (meshExportMode)
+	{
+		err = kernel->setArg(argIterator++,
+			*inCLConstMeshExportBuffer); // input data in constant memory (faster than global)
+		if (!checkErr(err, "kernel->setArg(3, *inCLConstMeshExportBuffer)"))
+		{
+			emit showErrorMessage(
+				QObject::tr("Cannot set OpenCL argument for %2").arg(QObject::tr("constant mesh data")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
+	}
+
+	if (renderEngineMode != clRenderEngineTypeFast && !meshExportMode)
 	{
 		err = kernel->setArg(
 			argIterator++, *backgroundImage2D); // input data in constant memory (faster than global)
@@ -1069,13 +1256,16 @@ bool cOpenClEngineRenderFractal::AssignParametersToKernelAdditional(int argItera
 		}
 	}
 
-	err = kernel->setArg(argIterator++, Random(1000000)); // random seed
-	if (!checkErr(err, "kernel->setArg(4, *inCLConstBuffer)"))
+	if (!meshExportMode)
 	{
-		emit showErrorMessage(
-			QObject::tr("Cannot set OpenCL argument for %3").arg(QObject::tr("random seed")),
-			cErrorMessage::errorMessage, nullptr);
-		return false;
+		err = kernel->setArg(argIterator++, Random(1000000)); // random seed
+		if (!checkErr(err, "kernel->setArg(4, *inCLConstBuffer)"))
+		{
+			emit showErrorMessage(
+				QObject::tr("Cannot set OpenCL argument for %3").arg(QObject::tr("random seed")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
 	}
 
 	return true;
@@ -1086,7 +1276,6 @@ bool cOpenClEngineRenderFractal::WriteBuffersToQueue()
 	cOpenClEngine::WriteBuffersToQueue();
 
 	cl_int err = queue->enqueueWriteBuffer(*inCLBuffer, CL_TRUE, 0, inBuffer.size(), inBuffer.data());
-
 	if (!checkErr(err, "CommandQueue::enqueueWriteBuffer(inCLBuffer)"))
 	{
 		emit showErrorMessage(
@@ -1104,8 +1293,30 @@ bool cOpenClEngineRenderFractal::WriteBuffersToQueue()
 		return false;
 	}
 
+	if (renderEngineMode == clRenderEngineTypeFull)
+	{
+		err = queue->enqueueWriteBuffer(
+			*inCLTextureBuffer, CL_TRUE, 0, inTextureBuffer.size(), inTextureBuffer.data());
+		if (!checkErr(err, "CommandQueue::enqueueWriteBuffer(inCLTextureBuffer)"))
+		{
+			emit showErrorMessage(
+				QObject::tr("Cannot enqueue writing OpenCL %1").arg(QObject::tr("input texture buffers")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
+
+		err = queue->finish();
+		if (!checkErr(err, "CommandQueue::finish() - inCLTextureBuffer"))
+		{
+			emit showErrorMessage(
+				QObject::tr("Cannot finish writing OpenCL %1").arg(QObject::tr("input texture buffers")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
+	}
+
 	err = queue->enqueueWriteBuffer(
-		*inCLConstBuffer, CL_TRUE, 0, sizeof(sClInConstants), constantInBuffer);
+		*inCLConstBuffer.data(), CL_TRUE, 0, sizeof(sClInConstants), constantInBuffer.data());
 	if (!checkErr(err, "CommandQueue::enqueueWriteBuffer(inCLConstBuffer)"))
 	{
 		emit showErrorMessage(
@@ -1123,6 +1334,28 @@ bool cOpenClEngineRenderFractal::WriteBuffersToQueue()
 		return false;
 	}
 
+	if (meshExportMode)
+	{
+		err = queue->enqueueWriteBuffer(*inCLConstMeshExportBuffer.data(), CL_TRUE, 0,
+			sizeof(sClMeshExport), constantInMeshExportBuffer.data());
+		if (!checkErr(err, "CommandQueue::enqueueWriteBuffer(inCLConstMeshExportBuffer)"))
+		{
+			emit showErrorMessage(QObject::tr("Cannot enqueue writing OpenCL %1")
+															.arg(QObject::tr("constant mesh export buffers")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
+
+		err = queue->finish();
+		if (!checkErr(err, "CommandQueue::finish() - inCLConstMeshExportBuffer"))
+		{
+			emit showErrorMessage(QObject::tr("Cannot finish writing OpenCL %1")
+															.arg(QObject::tr("constant mesh export buffers")),
+				cErrorMessage::errorMessage, nullptr);
+			return false;
+		}
+	}
+
 	return true;
 }
 
@@ -1172,9 +1405,17 @@ bool cOpenClEngineRenderFractal::ReadBuffersFromQueue()
 
 size_t cOpenClEngineRenderFractal::CalcNeededMemory()
 {
-	size_t mem1 = optimalJob.sizeOfPixel * optimalJob.stepSize;
-	size_t mem2 = dynamicData->GetData().size();
-	return max(mem1, mem2);
+	if (!meshExportMode)
+	{
+		size_t mem1 = optimalJob.sizeOfPixel * optimalJob.stepSize;
+		size_t mem2 = dynamicData->GetData().size();
+		return max(mem1, mem2);
+	}
+	else
+	{
+		size_t mem = 0; // TODO calculation of amout of needed memory
+		return mem;
+	}
 }
 
 bool cOpenClEngineRenderFractal::PrepareBufferForBackground(sRenderData *renderData)
@@ -1184,8 +1425,7 @@ bool cOpenClEngineRenderFractal::PrepareBufferForBackground(sRenderData *renderD
 	int texWidth = renderData->textures.backgroundTexture.Width();
 	int texHeight = renderData->textures.backgroundTexture.Height();
 
-	if (backgroungImageBuffer) delete[] backgroungImageBuffer;
-	backgroungImageBuffer = new cl_uchar4[texWidth * texHeight];
+	backgroungImageBuffer.reset(new cl_uchar4[texWidth * texHeight]);
 	size_t backgroundImage2DWidth = texWidth;
 	size_t backgroundImage2DHeight = texHeight;
 
@@ -1202,14 +1442,68 @@ bool cOpenClEngineRenderFractal::PrepareBufferForBackground(sRenderData *renderD
 	}
 
 	cl_int err;
-	if (backgroundImage2D) delete backgroundImage2D;
-	backgroundImage2D =
+	backgroundImage2D.reset(
 		new cl::Image2D(*hardware->getContext(), CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR,
 			cl::ImageFormat(CL_RGBA, CL_UNORM_INT8), backgroundImage2DWidth, backgroundImage2DHeight,
-			backgroundImage2DWidth * sizeof(cl_uchar4), backgroungImageBuffer, &err);
+			backgroundImage2DWidth * sizeof(cl_uchar4), backgroungImageBuffer.data(), &err));
 	if (!checkErr(err, "cl::Image2D(...backgroundImage...)")) return false;
 
 	return true;
 }
 
+bool cOpenClEngineRenderFractal::Render(double *distances, double *colors, int sliceIndex,
+	bool *stopRequest, sRenderData *renderData, size_t dataOffset)
+{
+	Q_UNUSED(renderData);
+	Q_UNUSED(stopRequest);
+
+	constantInMeshExportBuffer->sliceIndex = sliceIndex;
+
+	// writing data to queue
+	if (!WriteBuffersToQueue()) return false;
+
+	// assign parameters to kernel
+	if (!AssignParametersToKernel()) return false;
+
+	optimalJob.stepSizeX = constantInMeshExportBuffer->sliceWidth;
+	size_t pixelsLeftX = optimalJob.stepSizeX;
+
+	optimalJob.stepSizeY = constantInMeshExportBuffer->sliceHeight;
+	size_t pixelsLeftY = optimalJob.stepSizeY;
+
+	// processing queue
+	if (!ProcessQueue(0, 0, pixelsLeftX, pixelsLeftY)) return false;
+
+	if (!ReadBuffersFromQueue()) return false;
+
+	for (int y = 0; y < constantInMeshExportBuffer->sliceHeight; y++)
+	{
+		for (int x = 0; x < constantInMeshExportBuffer->sliceWidth; x++)
+		{
+			size_t address = x + y * constantInMeshExportBuffer->sliceWidth;
+			double distance =
+				(reinterpret_cast<cl_float *>(outputBuffers[outputMeshDistancesIndex].ptr.data()))[address];
+			distances[address + dataOffset] = distance;
+
+			if (colors)
+			{
+				double color =
+					(reinterpret_cast<cl_float *>(outputBuffers[outputMeshColorsIndex].ptr.data()))[address];
+				colors[address + dataOffset] = color;
+			}
+		}
+	}
+
+	return true;
+}
+
+void cOpenClEngineRenderFractal::SetMeshExportParameters(const sClMeshExport *meshParams)
+{
+	*constantInMeshExportBuffer = *meshParams;
+	if (meshParams->coloredMesh)
+	{
+		definesCollector += " -DMESH_EXPORT_COLOR";
+	}
+}
+
 #endif
