@@ -77,10 +77,10 @@ size_t cSettings::CreateText(const cParameterContainer *par, const cFractalConta
 
 	// standard parameters
 	QList<QString> parameterList = par->GetListOfParameters();
-	for (int i = 0; i < parameterList.size(); i++)
+	for (auto &parameterNameFromList : parameterList)
 	{
-		if (parameterList[i] == "description") continue;
-		settingsText += CreateOneLine(par, parameterList[i]);
+		if (parameterNameFromList == "description") continue;
+		settingsText += CreateOneLine(par, parameterNameFromList);
 	}
 
 	if (format != formatAppSettings)
@@ -91,9 +91,9 @@ size_t cSettings::CreateText(const cParameterContainer *par, const cFractalConta
 			{
 				QList<QString> parameterListFractal = fractPar->at(f).GetListOfParameters();
 				QString fractalSettingsText = "";
-				for (int i = 0; i < parameterListFractal.size(); i++)
+				for (const auto &parameterNameFromFractal : parameterListFractal)
 				{
-					fractalSettingsText += CreateOneLine(&fractPar->at(f), parameterListFractal[i]);
+					fractalSettingsText += CreateOneLine(&fractPar->at(f), parameterNameFromFractal);
 				}
 				if (fractalSettingsText.length() > 0)
 				{
@@ -506,7 +506,7 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 				}
 				else if (section.contains("fractal"))
 				{
-					int i = section.right(1).toInt() - 1;
+					int i = section.rightRef(1).toInt() - 1;
 					if (fractPar) result = DecodeOneLine(&fractPar->at(i), line);
 				}
 				else if (section == QString("frames"))
@@ -602,17 +602,17 @@ bool cSettings::Decode(cParameterContainer *par, cFractalContainer *fractPar,
 		if (keyframes && linesWithSoundParameters.length() > 0)
 		{
 			foundAnimSoundParameters = true;
-			for (int i = 0; i < linesWithSoundParameters.length(); i++)
+			for (const auto &linesWithSoundParameter : linesWithSoundParameters)
 			{
 				bool result;
-				result = DecodeOneLine(par, linesWithSoundParameters[i]);
+				result = DecodeOneLine(par, linesWithSoundParameter);
 
 				if (!result)
 				{
 					if (!quiet)
 					{
 						QString errorMessage =
-							QObject::tr("Error in settings file. Line: ") + linesWithSoundParameters[i];
+							QObject::tr("Error in settings file. Line: ") + linesWithSoundParameter;
 						cErrorMessage::showMessage(errorMessage, cErrorMessage::errorMessage);
 					}
 					errorCount++;
@@ -646,9 +646,8 @@ bool cSettings::CheckIfMaterialsAreDefined(cParameterContainer *par)
 {
 	bool matParameterFound = false;
 	QList<QString> list = par->GetListOfParameters();
-	for (int i = 0; i < list.size(); i++)
+	for (auto parameterName : list)
 	{
-		QString parameterName = list.at(i);
 		if (parameterName.left(3) == "mat")
 		{
 			matParameterFound = true;
@@ -704,6 +703,7 @@ bool cSettings::DecodeOneLine(cParameterContainer *par, QString line)
 			if (cMaterial::paramsList.indexOf(shortName) >= 0)
 			{
 				InitMaterialParams(matIndex, par);
+				PreCompatibilityMaterials(matIndex, par);
 				par->Set(QString("mat%1_is_defined").arg(matIndex), true);
 			}
 			else
@@ -777,7 +777,7 @@ bool cSettings::DecodeOneLine(cParameterContainer *par, QString line)
 
 bool cSettings::CheckSection(QString text, QString &section)
 {
-	if (text.left(1) == "[" && text.right(1) == "]")
+	if (text.left(1) == "[" && text.rightRef(1) == "]")
 	{
 		section = text.mid(1, text.length() - 2);
 		return true;
@@ -1109,12 +1109,12 @@ bool cSettings::DecodeFramesLine(
 			if (frameCount == frames->GetNumberOfFrames())
 			{
 				column++;
-				for (int i = 0; i < parameterList.size(); ++i)
+				for (auto &parameterDescription : parameterList)
 				{
 					using namespace parameterContainer;
-					enumVarType type = parameterList[i].varType;
-					QString containerName = parameterList[i].containerName;
-					QString parameterName = parameterList[i].parameterName;
+					enumVarType type = parameterDescription.varType;
+					QString containerName = parameterDescription.containerName;
+					QString parameterName = parameterDescription.parameterName;
 					cParameterContainer *container = frames->ContainerSelector(containerName, par, fractPar);
 
 					if (type == typeVector3)
@@ -1202,3 +1202,13 @@ QString cSettings::everyLocaleDouble(QString txt)
 	if (systemData.decimalPoint == '.') txtOut = txt.replace(',', '.');
 	return txtOut;
 }
+
+void cSettings::PreCompatibilityMaterials(int matIndex, cParameterContainer *par)
+{
+	if (fileVersion < 2.15)
+	{
+		par->Set(cMaterial::Name("metallic", matIndex), false);
+		par->Set(cMaterial::Name("specular", matIndex), 1.0);
+		par->Set(cMaterial::Name("specular_width", matIndex), 1.0);
+	}
+}
